import express from 'express';

import { Client, cacheExchange, fetchExchange, gql } from '@urql/core';
import { Response } from './types';

const app = express();
const PORT = 3000;


const audiothekApi = new Client({
  url: 'https://api.ardaudiothek.de/graphql',
  exchanges: [cacheExchange, fetchExchange],
});

const query = gql`
  query Item {
    item(id: "14278953") {
      show {
        coreId
        externalId
        title
        url
        publisher: publicationService {
          coreId
          dvbServiceId
          title
          url
          organizationName
          organization {
            name
            url
          }
        }
        coreDocument
      }
      assetId
      url: sharingUrl
      title
      description
      published: publishDate
      updated: core(key: "modified")
      imagesList {
        url
        title
      }
      coreDocument
      audioList {
        title
        audioCodec
        href
      }
    }
  }
`;

async function fetchData(query: any, variables: any) {
  const result = await audiothekApi.query<Response>(query, variables).toPromise();
  if (result.error) {
    throw result.error;
  }
  // console.log(JSON.stringify(result.data, null, 2));
  return result.data;
}

function transformEpisodeToActivityPub(response: Response) {
  const item = response.item;
  return {
    "@context": "https://www.w3.org/ns/activitystreams",
    "id": item.assetId,
    "type": "PodcastEpisode",
    "published": item.published,
    "updated": item.updated,
    "attributedTo": item.show.coreId,
    "externalId": item.url,
    "title": item.title,
    "description": {
      "type": "Note",
      "mediaType": "text/markdown",
      "content": item.description
    },
    "image": item.imagesList.length > 0 ? {
      "type": "Image",
      "url": item.imagesList[0].url
    } : undefined,
    "audio": item.audioList.map(audio => ({
      "type": "Audio",
      "name": audio.title,
      "mediaType": "audio/mpeg",
      "url": audio.href
    })),
    "partOf": {
      "type": "PodcastSeries",
      "id": item.show.coreId,
      "externalId": item.show.externalId,
      "name": item.show.title,
      "attributedTo": item.show.publisher.coreId,
      "publisher": {
        "type": "BroadcastService",
        "name": item.show.publisher.title,
        "url": item.show.publisher.url,
        "organization": {
          "type": "Organization",
          "name": item.show.publisher.organization.name,
          "url": item.show.publisher.organization.url
        }
      }
    }
  };
}

async function main() {
  const response = await fetchData(query, { id: '14278953' });

  if (response) {
    const activityPubEpisode = transformEpisodeToActivityPub(response);
    console.log(JSON.stringify(activityPubEpisode, null, 2));
  }
}

// main();


app.get('/:actor/episodes/:id', async (req, res) => {

  try {
    const data = await fetchData(query, { id: req.params.id });
    if (!data) {
      res.status(404).json({ error: 'Episode not found' });
      return;
    }

    // TODO: verifiy that the actor is the same as the attributedTo field

    const activityPubDocument = transformEpisodeToActivityPub(data);
    res.json(activityPubDocument);
  } catch (error) {
    res.status(500).json({ error: 'Failed to fetch episode data' });
  }
});

app.get('/@blaue-couch', async (req, res) => {
  res.json({
    "@context": "https://www.w3.org/ns/activitystreams",
    "id": "urn:ard:show:a42d1ea0b4a07053",
    "type": "Service",
    "name": "Blaue Couch",
    "externalId": "https://feeds.br.de/blaue-couch/feed.xml",
    "attributedTo": "urn:ard:publisher:c4a9cee041835529",
    "publisher": {
      "type": "BroadcastService",
      "name": "BAYERN 1",
      "url": "https://www.ardaudiothek.de/radio/br/bayern-1/",
      "organization": {
        "type": "Organization",
        "name": "BR",
        "url": "https://www.ardaudiothek.de/radio/br/"
      }
    }
  });
});

app.listen(PORT, () => {
  console.log(`Server running at http://localhost:${PORT}`);
});
